//    ___
//   |  _\ A free Lemmings clone for Linux           
//   |   /  _   _ _    ___    _   _   ___ 
//   |  |  | | | \ |  | _ |  | |_| | |_ -'
//   |__|  |_| |_\_|  |_  |  |_____| |___|
//                     _| |
//                    |___|
//
//   Copyright (C) 1998 Ingo Ruhnke <grumbel@gmx.de>
//     
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include <iostream>
#include <ctime>
#include <cstdio>
#include <unistd.h>
#include <ClanLib/clanlib.h>

#include "levelmap.hh"
#include "window.hh"
#include "playfield.hh"
#include "button_panel.hh"
#include "pingus.hh"
#include "pingu.hh"
#include "mouse.hh"
#include "globals.hh"

char* 
PingusMain::get_title()
{
  static char* title;
  
  if (!title) {
    title = new char[50];
    strcpy(title, "Pingus ");
    strcat(title, VERSION);
  }

  return title;
}

void 
PingusMain::check_args(int argc, char* argv[])
{
  pingus_data_dir = 0;
  cout << "Checking arguments..." << endl; 
  int c;
  opterr = 0;
  while ((c = getopt (argc, argv, "l:f:s:dp:")) != -1) {
    switch (c) {
    case 'l':
      strcpy(current_levelfile, optarg);
      current_level = -1;
      break;
    case 's':
      game_speed = atoi(optarg);
      break;
    case 'f':
      global_frame_rate = atoi(optarg);
      break;
    case 'd':
      use_dirty_tiles = true;
      break;
    case 'p':
      pingus_data_dir = strdup(optarg);
      break;
    case '?':
      cout << endl << "Pingus" << endl;
      cout << "~~~~~~" << endl;
      cout << "-l FILE    Load a custom level from FILE" << endl;
      cout << "-f INT     Sets the frame rate of the game" << endl;
      cout << "-d         Activate the new dirty tile mechanism" << endl;
      cout << "-s INT     Set the game speed" << endl; 
      cout << "-p DIR     Set the data path" << endl;
      cout << endl;
      exit(EXIT_FAILURE);
      break;
    }
  }
    
  cout << "Arguments correct" << endl;
}

void
PingusMain::intro(void)
{
}

// Loads a level from file and starts playing i
int
PingusMain::play_level(char* filename)
{
  PLF*         plf          = new PLF(filename);
  Window*      game         = new Window();

  // Create GuiObj for the Window game
  Playfield*   playfield    = new Playfield(plf);
  ButtonPanel* button_panel = new ButtonPanel();
  LevelMap*    level_map    = new LevelMap(plf);
  
  playfield->set_buttons(button_panel);

  game->add_obj(playfield);
  game->add_obj(button_panel);
  game->add_obj(level_map);
  game->let_move();

  delete          plf;
  delete    playfield;
  delete         game;
  delete button_panel;
  delete    level_map;
}


// Generates a filename from the level number and starts
// play_level(char*) with the filename.
int
PingusMain::play_level(int level_number)
{
  if (current_level != -1) {
    char* level_name;

    // Buffer overflow!
    level_name = new char[256];  // dirty, should use string instead
    
    sprintf(level_name, "%slevel%d.plf", pingus_data_dir, level_number);
    cout << level_name << endl;
    
    play_level(level_name);

    delete[] level_name;
  } else {
    play_level(current_levelfile);
  }
}

void
PingusMain::load_data()
{
  //  resource = new CL_ResourceManager(pingus_scriptfile, pingus_datafile);
  resource = new CL_ResourceManager(pingus_scriptfile);

  Pingu::load_data(resource);
  World::load_data(resource);
  Playfield::load_data(resource);
  ButtonPanel::load_data(resource);
  LevelMap::load_data(resource);
}
  
void
PingusMain::init(int argc, char* argv[])
{
  cout << "Pingus " << VERSION << " starting..." << endl;

  PingusMain::check_args(argc, argv);

  if (!pingus_data_dir)
    pingus_data_dir = getenv("PINGUS_DATA_DIR");

  if (!pingus_data_dir) {
    pingus_data_dir = strdup("/usr/local/share/pingus/");
  }

  pingus_datafile  = new char[255];
  pingus_scriptfile = new char[255];
  
  //cout << "Pingus Data File: " << pingus_datafile << endl;

  strcpy(pingus_datafile, pingus_data_dir);
  strcpy(pingus_scriptfile, pingus_data_dir);

  strcat(pingus_datafile, "pingus.dat");
  strcat(pingus_scriptfile, "pingus.scr");

  cout << "Pingus Data File: " << pingus_datafile << endl;
  cout << "Pingus Scrpit File: " << pingus_scriptfile << endl;

  // initialisation ClanLib
  CL_System::init_display();
  CL_Display::set_videomode(640, 480, 16);

  load_data();
  srand(time(NULL));
  
  if (!Mouse::init()) {
    cout << "No Mouse Support" << endl;
  }
}

int
PingusMain::main(int argc, char** argv)
{
  PingusMain::init(argc, argv);  
  PingusMain::play_level(3);

  return 0;
}

/* EOF */
